/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     Samuel Clement
 * Student ID: 9811498
 * Section:    AI
 *	
 *
 * Greeting Card Designer (Ultimate version)
 *
 *  The GP142 graphics window shows some buttons and the inside
 *  or outside (cover) of a greeting card.  The idea is that you 
 *  can design what you want to card to look like by adding and moving
 *  objects: 
 *    messages, message color and position
 *    shapes,   shape color and position
 *
 *   To place a message or a greeting, you click on an appropriate button, and 
 *   then you can click on the desired position in the card.  
 *
 *	 In this starter version, you can do much less.
 *   You can change the card background, place one shape on the card,
 *   and change the shape back and forth between two possible kinds.
 */

#include "gp142.h"
#include <math.h>
#include <time.h>

/*
 * General Constants
 */
#define TRUE    		1
#define FALSE   		0

/*
 * The state, or status you are in determines what happens when you click 
 * the card:
 * ADD SHAPE   -- you click the card, and a new object is placed there.
 *                You get into this mode by clicking an ADD button.
 *                After doing an ADD, you are put into SELECTED mode.
 * SELECTED    -- An object is selected. In this mode, the type 
 *                rotation applies to thecurrent object.
 * ADD MESSAGE -- adds a new message when you click the card, put in
 *                selected mode when you are done
 * SELECT NEW  -- select a new shape from the card for mainpulation
 */
#define STATUS_ADD_SHAPE		0
#define STATUS_SELECTED			1
#define STATUS_ADD_MESSAGE		2
#define STATUS_SELECT_NEW		3
#define MAX_STATUS				(STATUS_SELECT_NEW)
#define SELECT_RADIUS			20

/*
 * Button Constants
 */
#define BUTTON_HEIGHT  		35
#define BUTTON_WIDTH 		100
#define NUM_BUTTONS 		6
#define FACE3D_COLOR		SEA_GREEN
#define FACE3D_SHADOW		FOREST_GREEN
#define FACE3D_HIGHLIGHT	YELLOW
#define BUTTON_TEXT			NAVY_BLUE	
#define BUTTON_SPACE		10
#define HIGHLIGHT_COLOR		PURPLE
#define HIGHLIGHT_HIGHLIGHT	MAGENTA
#define HIGHLIGHT_SHADOW	DUSTY_PLUM
#define HIGHLIGHT_TEXT		YELLOW

/* 
 * position of the card in the window 
 */
#define PAGE_LLX			(-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY			(-GP142_YMAX + 120)
#define PAGE_WIDTH			(GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT			(GP142_YMAX - PAGE_LLY - 50)

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL		0
#define NOFILL		1

/* 
 * Types of objects we support.
 */
#define OBJECT_NONE			0
#define OBJECT_SHAPE		1
#define OBJECT_MESSAGE		2

#define MAX_OBJECTS			100
#define MAX_MESSAGES		90
#define MAX_SHAPES			90

/*
 * messages
 */
#define MESSAGE_FIRST		0
#define MESSAGE_LAST		10


/* 
 * These are the kinds of shapes we support.
 */
#define SHAPE_HEART			0
#define SHAPE_STAR			1
#define SHAPE_BEAR			2
#define SHAPE_TRI			3
#define LAST_SHAPE			4
#define FIRST_SHAPE_NUM		0


/* 
 * Possible places for mouse clicks
 */
#define CLICK_BUTTON		1	/* You clicked on SOME button (doesn't say which one)*/
#define CLICK_CARD			2   /* You clicked in the card area */
#define CLICK_OUTSIDE		3	/* You clicked outside the card area, not on a button*/	


#define HEART_WIDTH			35
#define STARPOINTS			5
#define STARRADIUS			40
#define PI					3.1416
#define TRITHINGY_HEIGHT	35
#define BEAR_R1				6
#define BEAR_R2				12
#define BEAR_R3				18


/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);
void DrawButtons(int last_button);
void DrawCard (int background_color, int status, int object_count, 
			   int* object_type, int* object_num, 
			   int* object_color, int* object_x, int* object_y);
void DrawShape(int color, int shape_type, int bottom_x, int bottom_y);
void DrawHeart(int x, int y, int color);
void DrawStar(int x, int y, int color);
void DrawBear(int x, int y, int color);
void DrawTriThingy(int x, int y, int color);


/* Mouse click handling functions */
void HandleMouseClick(int mouse_x, int mouse_y, int* last_button, int *background_color, 
					  int *status, int *object_count, int *object_type, 
					  int *object_num, int *object_color, 
					  int *object_x, int *object_y, int* selected_object,
					  int* message_count, int* shape_count );
int ClassifyMouseClick(int mouse_x, int mouse_y);
void HandleButton(int x, int y, int* last_button, int *background_color, int *status,
				  int object_count, int* object_type, int *object_num, int* object_color, int selected_object );

/* Add a shape to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			  int *object_type, int *object_num, int *object_color, 
			  int *object_x, int *object_y, int* message_count,
			  int background_color );

/* Add a message to the card */
void AddMessage( int x, int y, int* status, int* object_count, int* object_type,
				int* object_num, int* object_color, int* object_x, int* object_y,
				int* message_count, int background_color );
/* draw a message on the card */
void DrawMessage( int message_type, int color, int x, int y );
int GetSelectedObject( int x, int y, int* object_x, 
					  int* object_y, int object_count );

/*****************************************************************************
 *			  
 *		   MAIN
 *			   
 *****************************************************************************/

 /* main() has two chief purposes: to contain the important variables and 
 *initialize them, and to contain the "event loop," which constantly looks 
 *to see if an event has happened, and then processes the event by calling 
 *an appropriate function. An example of an event is a mouse click.
*/
int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
	
	
	/*variables and their default values*/
	int background_color = WHITE;
	int status = STATUS_ADD_SHAPE;
	
	/* 
	 * Each object will have a number telling what kind of object it is, a color,
	 * and an x and y coordinate.  You'll need to make these into arrays so you
	 * can support more than one object.
	 */
	int object_count = 0;           /* How many objects on the page? */
	int object_type[MAX_OBJECTS];    /* Can also be a shape           */
	int object_num[MAX_OBJECTS];    /* If type is shape */
	int object_color[MAX_OBJECTS];  /* Color of object               */
	int object_x[MAX_OBJECTS];      /* Position of object            */
	int object_y[MAX_OBJECTS];							
	int last_button = -1;           /* Keep track of the last button clicked.
									 * This would be usefull when highlighting
									 * the last button the user clicked */
	int event;						/*the most recent GP142 event*/
	int selected_object = -1;		/* currently selected object */
	int shape_count = 0;
	int message_count = 0;
	
	/*
	 * INITIALIZATIONS
	 */
	srand (time(NULL));	 	
	
	/* These GP142 initializations don't need to be changed */
	GP142_open();           /* Open and init the GP142 Graphics Window      */
	GP142_show_text(0);     /* Hide the text window                         */
	GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */
	
	GP142_clear();	        /* This will clear the window & paint it white. */
	GP142_animate(ANI_RUN); /* Enable periodic events                       */
	
	

	/*
 	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this until 
	 * the user signals "quit".
	 */
	while (!quit) 
	{  
		/*this is the famous "event loop" */
		
		/*start by redrawing everything */
		DrawBlankScreen ();
		DrawCard (background_color, status, object_count, 
			object_type, object_num, object_color, object_x, object_y);
		DrawButtons(last_button);	  /* Show the buttons...*/ 
		
		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		appropriately. */
		switch (event) 
		{
		case GP142_QUIT:
			quit = TRUE;           /* set flag to terminate loop           */
			break;
			
		case GP142_KBD:  /*A key was pressed */
						 /*not needed for this program. Use this if user will need to 
						 hit a key. Note that scanf CANNOT be used with GP142.  If you 
			use this, the key character value is in variable "key_hit"*/
			break;
			
		case GP142_MOUSE:
			/* 
			 * A mouse click has occurred. Call the mouse handler to 
			 * decide where the click was and react as needed. 
			 */
			HandleMouseClick( 
				mouse_x,
				mouse_y, 
				&last_button,
				&background_color, 
				&status, 
				&object_count, 
				object_type, 
				object_num,
				object_color, 
				object_x,
				object_y,
				&selected_object,
				&message_count,
				&shape_count );			
			break;
			
		case GP142_PERIODIC:
			/*An events has occurred simply because some time has passed,*/
			/*not currently used in the program*/		
			break;
			
		default:
			
			break;
			
		}  /*end switch */
		
	}  /* end event loop */
	
	
	/*
	 * WRAP-UP
	 */
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
} 

/*****************************************************************************
 * FUNCTIONS
 *
 *****************************************************************************/
/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen(void) 
{
	GP142_rectangleXY ( 
		BLACK,
		-GP142_XMAX, 
		-GP142_YMAX,
		GP142_XMAX,
		GP142_YMAX,
		FILL );
}
  
  
/* 
 * Draws the buttons 
 */
void DrawButtons(int last_button)
{
	char b_labels [NUM_BUTTONS][25] = 
	{
		/*  "012345678901234567890124" */
		"Card Color",
		"Object Color",
		"Insert Message",
		"Insert Object",
		"Select",
		"Next",
	};
	
	int text_offset = -GP142_XMAX + BUTTON_SPACE + 8;
	int font_size = 12;
	int i, y, x, label_y;
	int shadow_color;
	int highlight_color;
	int text_color;
	int face_color;
	
	/* draw rectangles and labels for each button */
	for ( i = 0; i < NUM_BUTTONS; i++ ) 
	{
		if ( last_button == i )
		{
			shadow_color = HIGHLIGHT_SHADOW;
			highlight_color = HIGHLIGHT_HIGHLIGHT;
			face_color = HIGHLIGHT_COLOR;
			text_color = HIGHLIGHT_TEXT;
		}
		else
		{
			shadow_color = FACE3D_SHADOW;
			highlight_color = FACE3D_HIGHLIGHT;
			face_color = FACE3D_COLOR;
			text_color = BUTTON_TEXT;
		}

		y = ( GP142_YMAX - BUTTON_SPACE ) - 
			( i * ( BUTTON_HEIGHT + BUTTON_SPACE ) );
		x = -GP142_XMAX + BUTTON_SPACE;
		
		/* fill the rectangle */
		GP142_rectangleXY( face_color, x, y, x + BUTTON_WIDTH, y - BUTTON_HEIGHT, FILL );
		
		/* draw our shadows */
		GP142_lineXY( shadow_color, x,	y - BUTTON_HEIGHT, x + BUTTON_WIDTH, y - BUTTON_HEIGHT, 1 );
		GP142_lineXY( shadow_color, x + BUTTON_WIDTH, y, x + BUTTON_WIDTH, y - BUTTON_HEIGHT, 1 );
		
		/* draw our highlights */
		GP142_lineXY( highlight_color,	x - 1, y, x + BUTTON_WIDTH,	y, 1 );
		GP142_lineXY( highlight_color, x, y, x, y - BUTTON_HEIGHT, 1 );
		
		label_y = y - ( BUTTON_HEIGHT / 2 ) - ( 2 * font_size / 3 );
		
		GP142_printfXY( text_color, text_offset, label_y, font_size, b_labels[i]);	
	}	
}
  
/*
 * Render (draw) the page and all its contents.
 */
void DrawCard(int background_color, int status, int object_count, 
				int* object_type, int* object_num, 
				int* object_color, int* object_x, int* object_y)
{
	
	char* userInfo[] = {
		"Click in the card to add a new shape",
		"Shape selected, 'Next' for different variants, 'Object color' to change color",
		"Click in the card to insert a message",
		"Click on an existing object to select",
	};
	int font_size = 16;
	int border_color = RED;
	int i;

	GP142_printfXY( YELLOW, PAGE_LLX, GP142_YMAX - 40, 
		18, "Greeting Card Designer" );

	/*
	* Draw the background of the card.
	*/
	GP142_rectangleXY (
		background_color,
		PAGE_LLX,
		PAGE_LLY,
		PAGE_LLX + PAGE_WIDTH,
		PAGE_LLY + PAGE_HEIGHT,
		FILL );
	
	if ( RED == background_color )
		border_color = GREEN;
	
	GP142_rectangleXY (
		border_color,
		PAGE_LLX,
		PAGE_LLY,
		PAGE_LLX + PAGE_WIDTH,
		PAGE_LLY + PAGE_HEIGHT,
		1 );

	
	/* 
 	 * Draw a shape on the card, if one currently exists.
	 */
	for ( i = 0; i < object_count; i++ )
	{
		if ( OBJECT_SHAPE == object_type[i] )
			DrawShape (object_num[i], object_color[i], object_x[i], object_y[i] );
		else if ( OBJECT_MESSAGE == object_type[i] )
			DrawMessage( object_num[i], object_color[i], object_x[i], object_y[i] );
	}
	
	/*
	 * Write an appropriate status message below the background
	 */
	if ( status > MAX_STATUS )
	{
		GP142_printfXY ( YELLOW, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer");
	}
	else
	{
		GP142_printfXY( status + PINK, -GP142_XMAX + 30, PAGE_LLY - 40, font_size, userInfo[status] );
	}	
}


/*
 * DrawMessage
 *
 * this draws a single message onot the card
 */
void DrawMessage( int message_type, int color, int x, int y )
{
	char* messages[MAX_MESSAGES] =
	{
		"Be Mine",
		"True Love",
		"Happy Valintines Day",
		"Be My Valintine",
		"Sweetie Pie",
		"XOXO",
		"Love You",
		"I'm Yours",
		"Hugs & Kisses",
		"Honey Bee"
	};

	int font_size = 12 + ( rand() % 4 );
	GP142_printfXY( color, x, y, font_size, messages[message_type] );
}
	 
/*
 * Draw one shape.
 */
void DrawShape(int shape_type, int color, int bottom_x, int bottom_y) 
{
	
	switch ( (shape_type - FIRST_SHAPE_NUM) % MAX_SHAPES ) 
	{
	case SHAPE_HEART:
		DrawHeart( bottom_x, bottom_y, color );
		break;

	case SHAPE_STAR:
		DrawStar( bottom_x, bottom_y, color );
		break;
	
	case SHAPE_BEAR:
		DrawBear( bottom_x, bottom_y, color );
		break;

	case SHAPE_TRI:
		DrawTriThingy( bottom_x, bottom_y, color );
		break;

	default:
		/* Unknown shape selected */
		break;
		
	} /*end switch*/
	
}

/*
 * DrawTriThingy
 *
 * this draws a Triangle thingy on the screen
 */
void DrawTriThingy( int x, int y, int color )
{
	int h = TRITHINGY_HEIGHT;
	int x1, x2, x3;
	int y1, y2, y3;
	int color2 = ( color + rand() % 4 ) % MAX_COLORS;
	int color3 = ( color + rand() % 5 ) % MAX_COLORS;
	int color4 = ( color + rand() % 6 ) % MAX_COLORS;

	/* calc our params */
	x1 = x - h / 2;
	x2 = x + h / 2;
	x3 = x;
	y1 = y + h / 2;
	y2 = y + h / 2;
	y3 = y - h / 2;

	GP142_triangleXY( color, x1, y1, x2, y2, x3, y3, FILL );
	GP142_triangleXY( color2, x1, y1, x1 - h / 2, y3, x1 + h / 2, y3, FILL );
	GP142_triangleXY( color3, x2, y1, x2 - h / 2, y3, x2 + h / 2, y3, FILL );
	GP142_triangleXY( color4, x1 + h / 2, y1 + h, x2, y1, x1, y1, FILL );
}

/*
 * DrawBear
 *
 * this draws a pulsating bear on the screen
 */
void DrawBear( int x, int y, int color )
{
	int r1 = BEAR_R1;
	int r2 = BEAR_R2;
	int r3 = BEAR_R3;
	int dx1, dx2, dx3;
	int puls;

	/*puls = rand() % 4;*/
	puls = 0;
	r1 += puls;
	r2 += puls;
	r3 += puls;
	dx1 = r2 / 2;
	dx2 = 2 * r3 / 3;
	dx3 = 3 * r3 / 4;

	/* draw the body */
	GP142_circleXY( color, x, y, r3 );
	GP142_circleXY( color, x - r1 - dx3, y + r3 / 3, r1 );
	GP142_circleXY( color, x + r1 + dx3, y + r3 / 3, r1 );
	GP142_circleXY( color, x - dx2, y - ( 7 * r3 / 8 ), r1 );
	GP142_circleXY( color, x + dx2, y - ( 7 * r3 / 8 ), r1 );

	/* draw the head */
	GP142_circleXY( color, x, y + ( 2 * r3 / 3 ) + r2 / 2, r2 );
	GP142_circleXY( color, x - dx1, y + 
		( 2 * r3 / 3 ) + ( 3 * r2 / 2 ), r1 );
	GP142_circleXY( color, x + dx1, y + 
		( 2 * r3 / 3 ) + ( 3 * r2 / 2 ), r1 );

}
	 
/*
 * Draw a heart on the card. A heart will be made by two overlapping
 * circles and a triangle.
 *
 * The pulsation is caused by adding a small, random amount to the 
 * heart height each time this function is called.
 */
void DrawHeart (int x, int y, int color) 
{
	
	int h = (HEART_WIDTH/2) + (rand () %4);
	
	GP142_circleXY (color, x - h + 5, y, h);
	GP142_circleXY (color, x + h - 5, y, h);
	GP142_triangleXY (color, x- h*2 +5, y-5,
		x + h*2 -5, y-5,
		x, y - h*2 -5, FILL);
	
}
	 
	 
/*
 * Draw a star on the card.
 *
 * We make the star twinkle by selecting the number of points randomly.  Each 
 * time it is redrawn, it is slightly different than the time before, which 
 * makes it appear to pulsate,
 */
void DrawStar (int x, int y, int color) 
{
	int sp;
	double angle;
	int x1, y1;
	
	sp = STARPOINTS + (rand () %3);
	for (angle = 0.0; angle < 2 * PI; angle = angle + (2*PI)/sp) 
	{
		x1 = (int) (cos (angle) * STARRADIUS);
		y1 = (int) (sin (angle) * STARRADIUS);
		GP142_lineXY (color, x+x1, y+y1, x-x1, y-y1, 1);		
	}
}
	 
	 
/*
 * Mouse clicks occur in one of three regions of interest: on buttons, 
 * on the card, or anywhere else.
 */
int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
	if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH) &&
		(mouse_y < GP142_YMAX) &&
		(mouse_y > (GP142_YMAX - NUM_BUTTONS*( BUTTON_HEIGHT + BUTTON_SPACE ) )))
	{
		return CLICK_BUTTON;
	}
	else if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
		&& mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
	{
		return CLICK_CARD;
	}
	
	return CLICK_OUTSIDE;
}

	 
/*
 * Decide where a mouse click occured, and react appropriately.
 */
void HandleMouseClick(int mouse_x, int mouse_y, int* last_button,
				int *background_color, 	int *status, int *object_count,
				int *object_type, int *object_num, int *object_color, 
				int *object_x, int *object_y, int* selected_object,
				int *message_count, int* shape_count )
{
	
	int mouse_click;
	int i, newSelect;

	mouse_click = ClassifyMouseClick(mouse_x, mouse_y);
	
	if (mouse_click == CLICK_BUTTON) 
	{
		/* the user has clicked on the card */
		HandleButton( 
			mouse_x, 
			mouse_y, 
			last_button,
			background_color, 
			status, 
			*object_count, 
			object_type, 
			object_num,
			object_color,
			*selected_object );
		
	} 
	else if (mouse_click == CLICK_CARD) 
	{
		
		if ( *status == STATUS_SELECT_NEW )
		{
			newSelect = GetSelectedObject( mouse_x, mouse_y, 
				object_x, object_y, *object_count );
			if ( newSelect >= 0 )
			{
				*selected_object = newSelect;
				*status = STATUS_SELECTED;
			}
		}
		else if ( *status == STATUS_SELECTED )
		{
			/* simply move the selected object */
			object_x[*selected_object] = mouse_x;
			object_y[*selected_object] = mouse_y;
		}
		else if ( *object_count < MAX_OBJECTS )
		{
			/* Add an object if we can add one*/
			i = *object_count;
			if ( *status == STATUS_ADD_SHAPE && *shape_count < MAX_SHAPES )
			{
				AddShape( mouse_x, mouse_y, status, object_count, 
							&object_type[i], &object_num[i], &object_color[i], 
							&object_x[i], &object_y[i], shape_count, *background_color );
			}
			if ( *status == STATUS_ADD_MESSAGE && *message_count < MAX_MESSAGES )
			{
				AddMessage( mouse_x, mouse_y, status, object_count,
							&object_type[i], &object_num[i], &object_color[i],
							&object_x[i], &object_y[i], message_count, *background_color  );
			}

			
			/* select the new object */
			*selected_object = i;
		}
	}
	/* Ignore any other mouse clicks */
}
	 

/*
 * GetSelectedObject
 *
 * returns the closet object to the points clicked. if we are further
 * away than "SELECT_RADIUS" then don't select anything,
 * returns -1 if there is nothing to select
 */
int GetSelectedObject( int x, int y, int* object_x, 
					  int* object_y, int object_count )
{
	int near_object = -1;
	double near_distance = -1;
	double distance;
	int i, dx, dy;

	for ( i = 0; i < object_count; i++ )
	{
		dx = abs( x - object_x[i] );
		dy = abs( y - object_y[i] );
		distance = sqrt( ( dx * dx ) + ( dy * dy ) );

		/* we are closer */
		if ( distance < near_distance || near_distance == -1 )
		{
			near_object = i;
			near_distance = distance;
		}
	}

	/* check we are within selection radius */
	if ( near_distance > SELECT_RADIUS )
		return  -1;

	return near_object;
}


/* 
 * Figure out which button was clicked, if any, and do the right 
 * thing.
 */
void HandleButton( int x, int y, int* last_button, int *background_color, int *status,
				int object_count, int* object_type, int *object_num,
				int* object_color, int selected_object )
{
	int buttonNum = 0;
	
	/* 
	* If we're outside of this area, we could not have clicked on
	* a button.
	*/
	if (x > -GP142_XMAX + BUTTON_WIDTH)  
		return ;	/*That's an error situation*/
	
	buttonNum = ( GP142_YMAX - y ) / ( BUTTON_HEIGHT + BUTTON_SPACE );	 

	switch(buttonNum) 
	{
	case 0:
		/* Change the background color */
		*background_color = (*background_color + 1) % MAX_COLORS;
		break;
		
	case 1:
		/* change the current object color */
		object_color[selected_object] = ( object_color[selected_object] + 1 ) % MAX_COLORS;
		if ( object_color[selected_object] == *background_color )
		{
			object_color[selected_object] = ( object_color[selected_object] + 1 ) % MAX_COLORS;
		}
		break;
		
	case 2:
		*status = STATUS_ADD_MESSAGE;
		break;

	case 3:
		*status = STATUS_ADD_SHAPE;
		break;

	case 4:
		*status = STATUS_SELECT_NEW;
		break;
		
	case 5:
		if ( *status != STATUS_SELECTED )
			return;

		/* Change the type of the currently selected object */
		if ( object_type[selected_object] == OBJECT_SHAPE )
		{
			object_num[selected_object] = (object_num[selected_object] + 1) % LAST_SHAPE;
		}
		else if ( object_type[selected_object] == OBJECT_MESSAGE )
		{
			object_num[selected_object] = (object_num[selected_object] + 1 ) % MESSAGE_LAST;
		}
		break;
		
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
		break;
	}

	*last_button = buttonNum;
}
	 
	 
/* If there's room for another shape, add it to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			 int *object_type, int *object_num, int *object_color, 
			 int *object_x, int *object_y, int* shape_count,
			 int background_color )
{

	/* make sure there is enough room for this object */
	if ( *object_count < MAX_OBJECTS ) 
	{
		 *status = STATUS_SELECTED;
		 (*object_count)++;
		 (*shape_count)++;

		 /* create a new shape */
		 *object_type = OBJECT_SHAPE;
		 *object_num = SHAPE_HEART;
		 *object_color = RED;
		/* make sure we don't match the background */
		if ( *object_color == background_color )
			*object_color = ( *object_color + 1 ) % MAX_COLORS;
		 *object_x = x;
		 *object_y = y;
	 }		 
}

/* 
 * AddMessage
 *
 * this adds a message to card.
 */
void AddMessage( int x, int y, int* status, int* object_count, int* object_type,
				int* object_num, int* object_color, int* object_x, int* object_y,
				int* message_count, int background_color )
{
	if ( *object_count < MAX_OBJECTS )
	{
		*status = STATUS_SELECTED;
		(*object_count)++;
		(*message_count)++;


		*object_type = OBJECT_MESSAGE;
		*object_num = MESSAGE_FIRST;
		*object_color = PURPLE;
		/* make sure we don't match the background */
		if ( *object_color == background_color )
			*object_color = ( *object_color + 1 ) % MAX_COLORS;
		*object_x = x;
		*object_y = y;
	}
}